@@ -6,13 +6,14 @@ import torch
 import torch.nn as nn
 import transformers
 
-from perceiver.model.core import EncoderConfig, InputAdapter, PerceiverEncoder
+from perceiver.model.core import EncoderConfig, InputAdapter, OutputAdapter, PerceiverEncoder
 from perceiver.model.core.convert import (
     copy_cross_attention_layer_params,
     copy_latent_provider_params,
     copy_params,
     copy_self_attention_block_params,
 )
+from perceiver.model.core.position import positions
 from perceiver.model.core.utils import freeze, is_checkpoint
 
 
@@ -38,9 +39,19 @@ class TextInputAdapter(InputAdapter):
     def max_seq_len(self):
         return self.pos_embedding.num_embeddings
 
-    def forward(self, x):
-        positions = torch.arange(0, x.shape[1], device=x.device)
-        return self.txt_embedding(x) + self.pos_embedding(positions)
+    def forward(self, x, abs_pos=None):
+        if abs_pos is None:
+            abs_pos = positions(*x.shape, device=x.device)
+        return self.txt_embedding(x) + self.pos_embedding(abs_pos)
+
+
+class TiedTextOutputAdapter(OutputAdapter):
+    def __init__(self, vocab_size: int):
+        super().__init__()
+        self.bias = nn.Parameter(torch.zeros(vocab_size))
+
+    def forward(self, x, txt_embedding: nn.Embedding):
+        return torch.matmul(x, txt_embedding.weight.T) + self.bias
 
 
 class TextEncoder(PerceiverEncoder):
